#!/pkg/ldc/bin/python2.1
"""
SelectPOS.py

This program lets the annotator select the right analysis for each word.

The following shell env. variables need to include the values on the right.

PYTHONPATH /pkg/ldc/lib
LD_LIBRARY_PATH /pkg/ldc/lib and /usr/openwin/lib

"""
import qt
from qt import *
import ag
import codecs
import sys
import re
import os
import string
import shutil
import getopt
if sys.platform == 'win32':
   import winsound

# set up the char set converters
(UTF8Encode, UTF8Decode, UTF8Reader, UTF8Writer) = codecs.lookup('utf-8')
(LEncode, LDecode, LReader, LWriter) = codecs.lookup('latin1')

def filterComment(s):
    return s

def stripVowel(line):
    return line

def cmpByPar(a, b):
    ap = int(ag.GetFeature(a, "paragraph"))
    bp = int(ag.GetFeature(b, "paragraph"))
    return cmp(ap, bp)

def parseText(s):
    """ Parse the input string for re-formatted text display. """
    return s

def parseFunc(s):
    """ Parse the input string for re-formatted text display. """
    return s

def utf8ize(s):
    return convertTim2Arabic(s)

def convertTim2Arabic(s):
    return s

    s = re.sub(',', u'\u060C', s)
    s = re.sub(';', u'\u061B', s)
    s = re.sub('\?', u'\u061F', s)

    s = re.sub('\'', u'\u0621', s)
    s = re.sub('\|', u'\u0622', s) 
    s = re.sub('\>', u'\u0623', s) 
    s = re.sub('\&', u'\u0624', s) 
    s = re.sub('\<', u'\u0625', s) 
    s = re.sub('\}', u'\u0626', s) 
    s = re.sub('A',  u'\u0627', s) 
    s = re.sub('b',  u'\u0628', s) 
    s = re.sub('p',  u'\u0629', s) 
    s = re.sub('t',  u'\u062a', s) 
    s = re.sub('v',  u'\u062b', s) 
    s = re.sub('j',  u'\u062c', s) 
    s = re.sub('H',  u'\u062d', s) 
    s = re.sub('x',  u'\u062e', s) 
    s = re.sub('d',  u'\u062f', s)

    s = re.sub('\*', u'\u0630', s)
    s = re.sub('r',  u'\u0631', s)
    s = re.sub('z',  u'\u0632', s)
    s = re.sub('s',  u'\u0633', s)
    s = re.sub('\$', u'\u0634', s)
    s = re.sub('S',  u'\u0635', s)
    s = re.sub('D',  u'\u0636', s)
    s = re.sub('T',  u'\u0637', s)
    s = re.sub('Z',  u'\u0638', s)
    s = re.sub('E',  u'\u0639', s)
    s = re.sub('g',  u'\u063a', s)

    s = re.sub('_',  u'\u0640', s)
    s = re.sub('f',  u'\u0641', s)
    s = re.sub('q',  u'\u0642', s)
    s = re.sub('k',  u'\u0643', s)
    s = re.sub('l',  u'\u0644', s)
    s = re.sub('m',  u'\u0645', s)
    s = re.sub('n',  u'\u0646', s)
    s = re.sub('h',  u'\u0647', s)
    s = re.sub('w',  u'\u0648', s)
    s = re.sub('Y',  u'\u0649', s)
    s = re.sub('y',  u'\u064a', s)
    s = re.sub('F',  u'\u064b', s)
    s = re.sub('N',  u'\u064c', s)
    s = re.sub('K',  u'\u064d', s)
    s = re.sub('a',  u'\u064e', s)
    s = re.sub('u',  u'\u064f', s)

    s = re.sub('i',  u'\u0650', s)
    s = re.sub('~',  u'\u0651', s)
    s = re.sub('o',  u'\u0652', s)

    s = re.sub('P',  u'\u067e', s)
    s = re.sub('J',  u'\u0686', s)
    s = re.sub('R',  u'\u0698', s)
    s = re.sub('G',  u'\u06af', s)

    if sys.platform == 'win32':
        s = re.sub('\(', 'ZZZZZZZZ', s)
        s = re.sub('\)', '(', s)
        s = re.sub('ZZZZZZZZ', ')', s)
        s = re.sub('\[', 'ZZZZZZZZ', s)
        s = re.sub('\]', '[', s)
        s = re.sub('ZZZZZZZZ', ']', s)

    s = re.sub('\/', '\\\\', s)

    return s

def filterArabic(s):
    return s

class SelectPOSMain(QMainWindow):
    """ Main application """
    def __init__(self):
        QMainWindow.__init__(self,
                             None,
                             'selectPOS',
                             Qt.WDestructiveClose)

        self.resize(900,800)
        self.setUpMain()
        self.setupMenuBar()

    def setUpMain(self):
        """ Set up the main widget. """
        self.main = SelectPOS(self, 'test')
        self.setCentralWidget(self.main)
        
    def setupMenuBar(self):
        """ Set up the menu bar. """
        
        self.file = QPopupMenu(self)
        self.menuBar().insertItem('&File', self.file)
        self.file.insertItem('Open', self.main.newLoad)
        self.file.insertItem('&Save', self.main.saveAIF, Qt.CTRL + Qt.Key_S)
        self.file.insertItem('&Quit',qApp,SLOT('closeAllWindows()'))

    def closeEvent(self,ce):
        """ Called when the program is being closed. """
        if self.main.saved:
            ce.accept()
            return

        rc = QMessageBox.information(self,'AG modified',
                                     'AG modified: do you want to save it?',
                                     'Save', 'Cancel','Leave Anyway',0,1)

        if rc == 0:
            self.main.saveAIF()
            ce.accept()
        elif rc == 2:
            ce.accept()
        else:
            ce.ignore()
            
class ArabicSelectPOSMain(SelectPOSMain):
    """ Subclass of SelectPOSMain for Arabic. """
    def setUpMain(self):
        self.main = ArabicSelectPOS(self, 'test')
        self.setCentralWidget(self.main)

class SelectPOS(QWidget):
    """ Main widget. """
    def __init__( self, *args ):
        apply(QWidget.__init__, (self, ) + args)

        ### GUI SECTION ###
        
        # create layouts
        
        self.allLayout = QHBoxLayout(self, 2)
        self.leftLayout = QVBoxLayout(10)
        self.rightLayout = QVBoxLayout(10)
	self.allLayout.addLayout(self.leftLayout, 2)
	self.allLayout.addLayout(self.rightLayout, 1)

        self.topLayout = QHBoxLayout()
        self.top2Layout = QGridLayout(5, 3)
        self.midLayout = QVBoxLayout()
        self.bottomLayout = QGridLayout(3, 9)

        self.leftLayout.addLayout(self.topLayout, 1)
        self.leftLayout.addLayout(self.top2Layout, 1)        
        self.leftLayout.addLayout(self.midLayout, 1)
        self.leftLayout.addLayout(self.bottomLayout, 1)

        # create font for text widgets
        font = QFont("clearlyu", 16, 50, 0)
	font_fixed = QFont("fixed", 12, 35, 0)

	# Right panel for dictionary
        self.search = QLineEdit(self, 'search')
	self.rightLayout.addWidget(self.search)
        self.checkDictionary = QPushButton(self, "SearchDictionary")
        self.checkDictionary.setText("Search Dictionary")
	self.rightLayout.addWidget(self.checkDictionary)
	self.connect(self.checkDictionary, SIGNAL("clicked()"), self.bindCheckDictionary)
	self.dictionary = QTextEdit(self, 'Dictionary')
        self.dictionary.setReadOnly(1)
	self.rightLayout.addWidget(self.dictionary)

        # main arabic orthgraphic display
        self.e = QTextEdit(self, 'editor')
        self.e.setFont(font)
        self.e.setReadOnly(1)
        self.topLayout.addWidget(self.e)

        self.rwb = QPushButton(self, "Prev")
        self.rwb.setText("Prev")
        self.lwb = QPushButton(self, "Next")
        self.lwb.setText("Next")                

        self.connect(self.lwb, SIGNAL("clicked()"), self.bindLeftWord)
        self.connect(self.rwb, SIGNAL("clicked()"), self.bindRightWord)

        self.unCommitB = QPushButton(self, "Uncommit Selection")
        self.unCommitB.setText("Remove Selection")
        self.connect(self.unCommitB, SIGNAL("clicked()"), self.bindRemoveSel)

        self.setCommentB = QPushButton(self, "Set Comment")
        self.setCommentB.setText("Set Comment")
        self.connect(self.setCommentB, SIGNAL("clicked()"), self.bindSetComment)

        self.noMatchB = QPushButton(self, "No match")
        self.noMatchB.setText("No match (<-)")
        self.connect(self.noMatchB, SIGNAL("clicked()"), self.bindNoMatch)

#        self.glossProblemB = QPushButton(self, "Gloss problem")
#        self.glossProblemB.setText("Gloss problem")
#        self.connect(self.glossProblemB, SIGNAL("clicked()"), self.bindGlossProblem)

#        self.typoB = QPushButton(self, "Typo")
#        self.typoB.setText("Typo")
#        self.connect(self.typoB, SIGNAL("clicked()"), self.bindTypo)

#        self.wrgB = QPushButton(self, "TRANSERR")
#        self.wrgB.setText("TRANSERR")
#        self.connect(self.wrgB, SIGNAL("clicked()"), self.bindWRG)

#        self.partialB = QPushButton(self, "Partial")
#        self.partialB.setText("Partial")
#        self.connect(self.partialB, SIGNAL("clicked()"), self.bindPartial)

#        self.changeVowelB = QPushButton(self, "ChangeVowel")
#        self.changeVowelB.setText("ChangeVowel")
#        self.connect(self.changeVowelB, SIGNAL("clicked()"), self.bindChangeVowel)

#        self.dialectalFormB = QPushButton(self, "Dialectal")
#        self.dialectalFormB.setText("Dialectal")
#        self.connect(self.dialectalFormB, SIGNAL("clicked()"), self.bindDialectalForm)

#        self.numericalB = QPushButton(self, "Number")
#        self.numericalB.setText("Number (<-)")
#        self.connect(self.numericalB, SIGNAL("clicked()"), self.bindNumerical)

#        self.punctuationB = QPushButton(self, "Punctuation")
#        self.punctuationB.setText("Punctuation (<-)")
#        self.connect(self.punctuationB, SIGNAL("clicked()"), self.bindPunctuation)

#        self.grammarB = QPushButton(self, "Grammar Problem")
#        self.grammarB.setText("Grammar Problem")
#        self.connect(self.grammarB, SIGNAL("clicked()"), self.bindGrammar)

        self.trans = QLineEdit(self, 'transliteration')
        self.trans.setReadOnly(1)
        self.transLabel = QLabel(self)
        self.transLabel.setText("Transliteration")
        self.top2Layout.addWidget(self.transLabel, 1, 1)
        self.top2Layout.addWidget(self.trans, 1, 2)

        self.commentLabel = QLabel(self)
        self.commentLabel.setText("Comment")
        self.comments = QLineEdit(self, 'comments')
        self.top2Layout.addWidget(self.commentLabel, 2, 1)
        self.top2Layout.addWidget(self.comments, 2, 2)

        self.docLabel = QLabel(self)
        self.docLabel.setText("Annotation File")

        self.doc = QLineEdit(self, 'docID')
        self.doc.setReadOnly(1)                
        self.top2Layout.addWidget(self.docLabel, 3, 1)
        self.top2Layout.addWidget(self.doc, 3, 2)

        self.blockLabel = QLabel(self)
        self.blockLabel.setText("Paragraph Number")
        self.block = QLineEdit(self, 'block')
        self.block.setReadOnly(1)                      
        self.top2Layout.addWidget(self.blockLabel, 4, 1)
        self.top2Layout.addWidget(self.block, 4, 2)

        self.selectedLabel = QLabel(self)
        self.selectedLabel.setText("Selected Analysis")
        self.selected = QLineEdit(self, 'selected')
        self.selected.setReadOnly(1)                                
        self.top2Layout.addWidget(self.selectedLabel, 5, 1)
        self.top2Layout.addWidget(self.selected, 5, 2)                

        self.formattedLabel = QLabel(self)
        self.formattedLabel.setText("Formatted Selection")
        self.formatted = QLineEdit(self, 'selected')
        self.formatted.setReadOnly(1)                                
        self.top2Layout.addWidget(self.formattedLabel, 6, 1)
        self.top2Layout.addWidget(self.formatted, 6, 2)                

        self.lb = QListBox(self, 'listbox')
	self.lb.setFont(font_fixed)
        self.connect(self.lb, SIGNAL("selected(int)"), self.bindAnalysisSelected)        

        self.midLayout.addWidget(self.lb)

        self.prev = QPushButton(self, "Prev")
        self.prev.setText("Prev. Paragraph")        

        self.next = QPushButton(self, "Next")
        self.next.setText("Next Paragraph")

        self.goto = QPushButton(self, "Goto")
        self.goto.setText("Goto Paragraph")

        self.connect(self.next, SIGNAL("clicked()"), self.bindNextBlock)
        self.connect(self.prev, SIGNAL("clicked()"), self.bindPrevBlock)
        self.connect(self.goto, SIGNAL("clicked()"), self.bindGotoBlock)

#        self.setXB = QPushButton(self, "X-Solution")
#        self.setXB.setText("X-Solution")
#        self.connect(self.setXB, SIGNAL("clicked()"), self.bindSetX)

        self.addPOS = QPushButton(self, "Add-POS")
        self.addPOS.setText("Add-POS")
        self.connect(self.addPOS, SIGNAL("clicked()"), self.bindAddPOS)

#        self.bottomLayout.addWidget(self.setXB, 0, 0)
        self.bottomLayout.addWidget(self.prev, 1, 0)
        self.bottomLayout.addWidget(self.goto, 1, 1)
        self.bottomLayout.addWidget(self.next, 1, 2)
        self.bottomLayout.addWidget(self.lwb, 0, 1)
        self.bottomLayout.addWidget(self.rwb, 0, 0)
        self.bottomLayout.addWidget(self.noMatchB, 0, 3)
        self.bottomLayout.addWidget(self.setCommentB, 0, 2)
#        self.bottomLayout.addWidget(self.glossProblemB, 2, 4)
#        self.bottomLayout.addWidget(self.typoB, 1, 5)
#        self.bottomLayout.addWidget(self.partialB, 1, 3)
#        self.bottomLayout.addWidget(self.changeVowelB, 3, 2)
        self.bottomLayout.addWidget(self.unCommitB, 1, 3)
#        self.bottomLayout.addWidget(self.numericalB, 0, 7)
#        self.bottomLayout.addWidget(self.dialectalFormB, 1, 7)
#        self.bottomLayout.addWidget(self.punctuationB, 0, 8)
#        self.bottomLayout.addWidget(self.grammarB, 1, 8)
        self.bottomLayout.addWidget(self.addPOS, 0, 8)
 
#        self.bottomLayout.addWidget(self.wrgB, 1, 4)

#        self.foreignB = QPushButton(self, "FOREIGN")
#        self.foreignB.setText("FOREIGN")
#        self.connect(self.foreignB, SIGNAL("clicked()"), self.bindForeign)

#        self.bottomLayout.addWidget(self.foreignB, 0, 6)

#        self.disflB = QPushButton(self, "DISFL")
#        self.disflB.setText("DISFL")
#        self.connect(self.disflB, SIGNAL("clicked()"), self.bindDisfl)

#        self.bottomLayout.addWidget(self.disflB, 1, 6)

        self.allLayout.activate()

        ### AG SECTION ###
        
        self.aifFilename = None
        self.textFilename = None
        self.outputFilename = None

        self.saved = 1
        self.currentAGIndex = 0
        self.currentTextList = ()
        self.currentAG = None
	self.x = 0
	self.forceStop = 1
	self.audioPID = 0
	self.currentAnn = None
	self.soundSegment = None
	self.dictionaryHash = None

    def bindCheckDictionary(self):
	if self.dictionaryHash == None:
	    self.loadDictionary()
#	print "ZZZZZZZZZZZZZZZ", self.dictionaryHash["mali:ntsi:n"]
#	print "ZZZZZZZZZZZZZZZ", self.dictionaryHash["i:xte:nkoya:wi"]
	w = self.search.text().latin1()
	w = re.sub('\s', '', w)
	try:
	    dict = self.dictionaryHash[w]
	except:
	    dict = None
	self.dictionary.clear()
	if dict != None:
	    self.dictionary.setText(dict)
	return

    def loadDictionary(self):
        fn = QFileDialog.getOpenFileName(QString.null,QString.null,self)
        if fn.isEmpty():
            return
	fileName = str(fn)
	dictFile = open(fileName, 'r')

	self.dictionaryHash = {}
	n = 0
	ts = []
	content = ''
	while 1:
	    proc = 0
	    line = dictFile.readline()
	    # print n, line
            if not line:
                break
	    if line[:5] == '\\ref ':
		n += 1
		if n == 100:
		    # print "100 more ", n
		    n = 0
		if len(ts) > 0:
		    # print "Keys", ts
		    # print "Dict", content
		    for f in ts:
			try:
			    if self.dictionaryHash[f][:len(content)] != content:
				if self.dictionaryHash[f][-len(content):] != content:
				    self.dictionaryHash[f] = self.dictionaryHash[f]+'\n'+content
			except:
			    self.dictionaryHash[f] = content
		    # print "IN", f, self.dictionaryHash[f]
		    ts = []
		    content = ''
		proc = 1

	    if line[:6] == '\\lxoa ':
		line = re.sub('[\n\r]', '', line)
		ts += [line.split(' ')[1]]
		# print "XXXX", ts
		proc = 1
	    if line[:8] == '\\lxoa_c ':
		line = re.sub('[\n\r]', '', line)
		ts += [line.split(' ')[1]]
		# print "XXXX", ts
		proc = 1
	    if line[:10] == '\\lxoa_alt ':
		line = re.sub('[\n\r]', '', line)
		ts += [line.split(' ')[1]]
		# print "XXXX", ts
		proc = 1
	    if line[:9] == '\\lxoa_pr ':
		line = re.sub('[\n\r]', '', line)
		ts += [line.split(' ')[1]]
		# print "XXXX", ts
		proc = 1
	 
	    if proc == 0:
		if line[:1] == '\\':
		    content += line[1:]

	dictFile.close()
	if len(ts) > 0:
	    for f in ts:
		try:
		    self.dictionaryHash[f] = self.dictionaryHash[f]+'\n'+content
		except:
		    self.dictionaryHash[f] = content
	# print self.dictionaryHash.keys()
	return

    def bindPlaySegment(self):
	return

    def bindStopSegment(self):
	return

    def newLoad(self):
        """ Load file from the menu item. """
        if self.aifFilename:
            rc = QMessageBox.information(self,'AG open',
                                         'AIF already open.  Please restart the'+
                                         ' application to work on another file.',
                                         'OK')
            return

        fn = QFileDialog.getOpenFileName(QString.null,QString.null,self)
        if fn.isEmpty():
            return
        
        self.aifFileName = str(fn)
        self.loadAIFFile(self.aifFileName)

    def bindAddPOS(self):
	w = ag.GetFeature(self.currentAnn, 'lookup-word')
	b = "Please enter"
	text = QInputDialog.getText("Input analysis", "Add a morphology analysis for word: "+w, QLineEdit.Normal, b)

	if text[0] == None:
	    return
	if text[0].latin1() == None:
	    return
	if text[0].latin1() == b:
	    return
	if text[0].latin1() == "NO_SELECTION":
	    return

	b = re.sub(r'[\s\t]', '', text[0].latin1())
	f = self.addSelection(b)
	return

    def addSelection(self, analysis):
	if analysis == None:
	    return 0
	if analysis == '':
	    return 1
	newID = 1;
	ann = self.currentAnn
	if analysis != None:
	    s = ag.GetStartAnchor(ann)
	    e = ag.GetEndAnchor(ann)
	    localAnnSet = ag.GetOutgoingAnnotationSet(s).split()
	    for thisAnn in localAnnSet:
                if ag.GetAnnotationType(thisAnn) == "solution":
                    newID += 1
		    if ag.GetFeature(thisAnn, "solution") == analysis:
			return 2
	if 1:
	    annSolID = ag.CreateAnnotation(ann, s, e, 'solution')
	    ag.SetFeature(annSolID, 'solution', analysis)
	    ag.SetFeature(annSolID, 'number', str(newID))
	    self.currentAnalysesDict[ann] += [annSolID]
	    self.insertAnalyses()
	    return annSolID
	else:
	    return 3
	return

    def bindNoMatch(self):
        """ Bound to the no match button. """
        try:
            comment = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comment = None
            
	if comment == "":
	    comment = None

        self.comments.clear()
        
        if comment is None:
            comment = "NO_MATCH"
        else:
            comment += "; NO_MATCH"

        self.comments.setText(comment)
        ag.SetFeature(self.currentAnn, 'comment', comment)
        self.selected.clear()
        self.formatted.clear()
        ag.SetFeature(self.currentAnn, 'selection', 'NONE_OF_THE_ABOVE')
        self.selected.setText('NONE_OF_THE_ABOVE')
        self.formatted.setText('NONE_OF_THE_ABOVE')
	self.bindLeftWord()

    def bindNotInSpeech(self):
        """ Bound to the Not_in_Speech button. """
	return

    def bindPassiveForm(self):
        """ Bound to the passive form button. """
	return

    def bindGlossProblem(self):
        """ Bound to the gloss problem button. """
        try:
            comment = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comment = None
            
	if comment == "":
	    comment = None

        self.comments.clear()
        
        if comment is None:
            comment = "PROBLEM_WITH_GLOSS"
        else:
            comment += "; PROBLEM_WITH_GLOSS"
        self.comments.setText(comment)
        ag.SetFeature(self.currentAnn, 'comment', comment)
	#self.bindLeftWord()

    def bindWCE(self):
        """ Bound to the WCE button. """
	return

    def bindNCE(self):
        """ Bound to the NCE button. """
	return

    def bindRegAccent(self):
        """ Bound to the REG_ACC button. """
	return

    def bindWordMissingBefore(self):
        """ Bound to the W_Missing_before button. """
	return

    def bindWordMissingAfter(self):
        """ Bound to the W_Missing_before button. """
	return

    def bindShouldBeU(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeA(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeI(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeF(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeK(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeN(self):
        """ Bound to the should be button. """
	return

    def bindShouldBeH(self):
        """ Bound to the should be button. """
	return

    def bindHamzaProblem(self):
        """ Bound to the hamza problem button. """
	return

#    def bindTypo(self):
#        """ Bound to the typo button. """
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment')
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "TYPO"
#        else:
#            comment += "; TYPO"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#	#self.bindLeftWord()
#
#        try:
#            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
#        except:
#	    print "Can't find look-up word"
#            lookup = None
#
#	if lookup != None:
#	    self.addAG('', 'typo', lookup + '/TYPO', 'NO_GLOSS')
#
#    def bindVOC(self):
#        """ Bound to the VOC_VAR button. """
#	return
#
#    def bindWRG(self):
#        """ Bound to the TRANSERR button. """
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment')
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "TRANSERR"
#        else:
#            comment += "; TRANSERR"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#	#self.bindLeftWord()
#
#        try:
#            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
#        except:
#	    print "Can't find look-up word"
#            lookup = None
#
#	if lookup != None:
#	    self.addAG('', 'transerr', lookup + '/TRANSERR', 'NO_GLOSS')
#
#    def bindPartial(self):
#        """ Bound to the partial button. """
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment') 
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "PARTIAL_WORD"
#        else:
#            comment += "; PARTIAL_WORD"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#	#self.bindLeftWord()
#
#        try:
#            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
#        except:
#	    print "Can't find look-up word"
#            lookup = None
#
#	if lookup != None:
#	    self.addAG('', 'partial', lookup + '/PARTIAL', 'NO_GLOSS')
#
#    def bindDisfl(self):
#        """ Bound to the disfl button. """
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment')
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "DISFL"
#        else:
#            comment += "; DISFL"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#	#self.bindLeftWord()
#
#        try:
#            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
#        except:
#	    print "Can't find look-up word"
#            lookup = None
#
#	if lookup != None:
#	    self.addAG('', 'disfl', lookup + '/DISFL-IP', 'NO_GLOSS')
#
#    def bindForeign(self):
#        """ Bound to the foreign button. """
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment')
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "FOREIGN"
#        else:
#            comment += "; FOREIGN"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#	#self.bindLeftWord()
#
#        try:
#            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
#        except:
#	    print "Can't find look-up word"
#            lookup = None
#
#	if lookup != None:
#	    self.addAG('', 'foreign', lookup + '/FOREIGN', 'NO_GLOSS')

    def addAG(self, ws, lemma, pos, gloss):
	return

    def bindClitic(self):
        """ Bound to the Clitic button. """
	return

    def bindDialectalForm(self):
        """ Bound to the Dialectal Form button. """
        try:
            comment = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comment = None
            
	if comment == "":
	    comment = None

        self.comments.clear()
        
        if comment is None:
            comment = "DIALECTAL_FORM"
        else:
            comment += "; DIALECTAL_FORM"
        self.comments.setText(comment)
        ag.SetFeature(self.currentAnn, 'comment', comment)
	#self.bindLeftWord()

    def bindNumerical(self):
        """ Bound to the Numerical button. """
        try:
            comment = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comment = None
            
	if comment == "":
	    comment = None

        self.comments.clear()
        
        if comment is None:
            comment = "NUMERICAL"
        else:
            comment += "; NUMERICAL"
        self.comments.setText(comment)
        ag.SetFeature(self.currentAnn, 'comment', comment)
	self.bindLeftWord()

    def bindPunctuation(self):
        """ Bound to the punctuation button. """
        try:
            comment = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comment = None
            
	if comment == "":
	    comment = None

        self.comments.clear()
        
        if comment is None:
            comment = "PUNCTUATION"
        else:
            comment += "; PUNCTUATION"
        self.comments.setText(comment)
        ag.SetFeature(self.currentAnn, 'comment', comment)
	self.bindLeftWord()

    def bindGrammar(self):
        """ Bound to the punctuation button. """
	return

    def bindNoun2Adj(self):
        """ Bound to the NOUN to ADJ button. """
	return

    def bindAdj2Noun(self):
        """ Bound to the ADJ to NOUN button. """
	return

#    def bindChangeVowel(self):
#        """ Bound to the ChangeVowel button. """
#
#	index = self.lb.currentItem()
#	if index == None:
#	    return
#
#	idx = self.refTable[index]
#	selectedAnalysisAnn = self.currentAnalysesDict[self.currentAnn][idx]
#	try:
#	    gloss = ag.GetFeature(selectedAnalysisAnn, 'gloss')
#	except:
#	    gloss = ''
#	sol = ag.GetFeature(selectedAnalysisAnn, 'solution')
#	(ws, lemma, pos) = sol.split(' ')
#	
#	text = QInputDialog.getText("ChangeVowel", pos, QLineEdit.Normal, pos)
#	if text[0] == None:
#	    return
#	if text[0].latin1() == None:
#	    return
#	if text[0].latin1() == pos:
#	    return
#	if parseText(text[0].latin1()) == '':
#	    return
#
#	oldT = parseFunc(pos)
#	newT = parseFunc(text[0].latin1())
#	oldC = stripVowel(parseText(pos))
#	newC = stripVowel(parseText(text[0].latin1()))
#
#	if oldT == 'NUM':
#	    return
#	if oldT == 'PUNC':
#	    return
#
#	oldC = re.sub('0', '', oldC)
#	newC = re.sub('0', '', newC)
#
#	# print "Old", oldT, oldW
#	# print "New", newT, newW
#
#	if oldT != newT:
#	    return
#	if oldC != newC:
#	    return
#
#        try:
#            comment = ag.GetFeature(self.currentAnn, 'comment')
#        except:
#            comment = None
#            
#	if comment == "":
#	    comment = None
#
#        self.comments.clear()
#        
#        if comment is None:
#            comment = "VOWEL_TO_BE_CHANGED"
#        else:
#            comment += "; VOWEL_TO_BE_CHANGED"
#        self.comments.setText(comment)
#        ag.SetFeature(self.currentAnn, 'comment', comment)
#
#	self.addAG('', lemma, text[0].latin1(), gloss)
#	#self.bindLeftWord()

    def bindRemoveCase(self):
        """ Bound to the rm CASE button. """
	return

    def bindNoun2Adv(self):
        """ Bound to the NOUN to ADV button. """
	return

    def bindEmailReport(self):
	""" Bound to the email report buttom. """
	return

    def bindRemoveSel(self):
        """ BOund to the remove selection button. """ 

	if self.currentAnn == None:
	    return

        try:
            ag.DeleteFeature(self.currentAnn, 'selection')
            self.saved = None
        except:
            pass
        self.selected.clear()
        self.formatted.clear()

    def bindAnalysisSelected(self, index):
        """ Bound to selet analysis event (i.e., double click) """
	idx = self.refTable[index]
	#print self.refTable
	#print index
	#print idx
        selectedAnalysisAnn = self.currentAnalysesDict[self.currentAnn][idx]
        ag.SetFeature(self.currentAnn, 'selection', selectedAnalysisAnn)
        self.saved = None
        self.selected.clear()
        self.selected.setText(self.lb.currentText())
        #print ag.GetFeature(selectedAnalysisAnn, 'solution')
        #print self.lb.currentText()
	self.bindLeftWord()

    def bindSetX(self):
        """ Bound to the set X-solution button. """
	if self.x == 1:
	    self.x = 0
	else:
	    self.x = 1
	self.insertAnalyses()

    def bindSetComment(self):
        """ Bound to the set comment button. """

	if self.currentAnn == None:
	    return

        comment = self.comments.text()
        ag.SetFeature(self.currentAnn, 'comment', str(comment))
        self.saved = None

    def saveAIF(self):
        """ Save the current AG as an AIF File. """
        
        #aifFilename = 'tmp.out'
        if self.outputFilename:
            aifFilename = self.outputFilename
        else:
            aifFilename = self.aifFilename

        fName = os.path.split(aifFilename)[1]
        dirName = os.path.split(aifFilename)[0]

        backupDirName = os.path.join(dirName, 'BACKUPS')
        for i in range(40):
            bFile = fName + '.backup_' + str(i)
            backupFile = os.path.join(backupDirName, bFile)
            try:
                os.stat(backupFile)
            except:
                # file does not exist --- so use this name
                try:
                    shutil.copyfile(self.aifFilename, backupFile)
                    print 'Backup file made:', backupFile
                except:
                    print 'Backup file not made'
                    pass
                break

        print "Saving AG to ", aifFilename
	#ag.store("AIF", aifFilename)
        out = LDecode(ag.toXML(self.agSetID))[0]
        aifFile = UTF8Writer(open(aifFilename, 'w'))
        aifFile.write(out)
        aifFile.close()
        self.saved = 1
        
    def loadAIFFile(self, filename):
        """ Load an annotation file in the AIF (xml) format.
        Find the text file which the AIF format refers to, and
        open it as well.
        """

        # The following used relative path from the location of the
        # xml file to find the text file --- but now it uses the
        # signal href in the xml file --- see below.
        #fileStem = string.join(os.path.split(filename)[-1].split('.')[:-1],
        #'.')
        #dir = os.path.split(os.path.split(filename)[0])[0]
        #textFilename = os.path.join(dir, 'sgm', fileStem+'.sgm')

        self.aifFilename = filename

        # This could be something like: 
        #"DATA/20000715_AFP_ARB/sgm/20000715_AFP_ARB.0002.sgm")

        # load the AIF file
        self.agIDstr = ag.load("AIF", filename)
        self.agIDs = self.agIDstr.split()
        self.agIDs.sort(cmpByPar)
        self.agSetID = ag.GetAGSetId(self.agIDs[0])
        self.timelineID = ag.GetTimelineId(self.agIDs[0])
        self.signalID = ag.GetSignals(self.timelineID).split()[0]

        # Then get the signal href
        textFilename = ag.GetSignalXlinkHref(self.signalID)

        # And load the text file.
        self.currentTextList = self.loadTextFile(textFilename)
        self.textFilename = textFilename

        # insert the name of the aif file
        self.doc.setText(self.aifFilename)

        # start from 0
        self.currentAGIndex = 0
        self.processBlock(self.currentAGIndex)
        
    def processBlock(self, index):
        """ Process one paragraph. """
        self.block.clear()
        self.block.setText(str(self.currentAGIndex+1))
        self.currentAG = self.agIDs[self.currentAGIndex]
        self.e.clear()
        self.displayText(self.currentTextList, index)
        self.processAG(self.currentAG)
        
    def processAG(self, agID):
        """ Process one AG. """ 
        annSet = ag.GetAnnotationSeqByOffset(agID).split()
        self.currentAnnSet = []
        self.currentAnnDict = {}
        self.currentAnalysesDict = {}
        wordAnnSet = []
        
        for ann in annSet:
            if ag.GetAnnotationType(ann) == "word":
                self.currentAnnSet.append(ann)
                startOffset = ag.GetStartOffset(ann)
                endOffset = ag.GetEndOffset(ann)
                self.currentAnnDict[ann] = (startOffset, endOffset)
		# print ann, startOffset, endOffset
                wordAnnSet.append(ann)
                
        for wordAnn in wordAnnSet:
            startAnchor = ag.GetStartAnchor(wordAnn)
            localAnnSet = ag.GetOutgoingAnnotationSet(startAnchor).split()

            analyses = []            
            for ann in localAnnSet:
                if ag.GetAnnotationType(ann) == "solution":
                    analyses.append(ann)
            self.currentAnalysesDict[wordAnn] = analyses

        # start from the first word
	if len(wordAnnSet) > 0:
            self.currentAnn = self.currentAnnSet[0]
            self.processWord()
	else:
	    self.selected.clear()   
	    self.formatted.clear()     

    def processWord(self):
        """ Process one word using the self.currentAnn ID. """
        (s, e) = self.currentAnnDict[self.currentAnn]
        self.selected.clear()        
        self.formatted.clear()        
        try:
            sel = ag.GetFeature(self.currentAnn, 'selection')
        except:
            sel = None
        if sel:
            if sel == "NONE_OF_THE_ABOVE":
                string = sel
            else:
                string = ag.GetFeature(sel, 'solution')
                try:
                    string += " : "+ag.GetFeature(sel, 'gloss')
                except:
                    string += ' :  "NO_GLOSS"'
	    self.selected.setText(string)
	    newstring = self.parseText(string)
	    self.formatted.setText(newstring)
        self.highlightText(0, s, e)
        self.insertAnalyses()

    def highlightText(self, line, start, end):
        """ Highlight a region in the text display. """
        s = int(start)
        e = int(end)
        self.e.setSelection(line, int(s), line, int(e))

    def bindLeftWord(self):
        """ Bound to the left buttom. """ 

	if self.forceStop == 1:
	    try:
		s = ag.GetFeature(self.currentAnn, 'selection')
		if s == None:
		    QMessageBox.information(self,'No selection',
                                         'Must have a selection, or NON_OF_THE_ABOVE', 'OK')
		    return
		s = re.sub('\s', '', s)
		if s == '':
		    QMessageBox.information(self,'No selection',
                                         'Empty string is not valid', 'OK')
		    return
	    except:
		QMessageBox.information(self,'No selection',
                                         'Must have a selection, or NON_OF_THE_ABOVE', 'OK')
		return

	self.bindSetComment()
	self.x = 0     

#	self.ne.clear()
        index = self.currentAnnSet.index(self.currentAnn)
        try:
            self.currentAnn = self.currentAnnSet[index+1]
        except:
            return

	self.block.clear()
	string = str(self.currentAGIndex+1)
	string += " out of total "
	string += str(len(self.agIDs))
	string += "    (word "
	string += str(index+2)
	string += " )"
	self.block.setText(string)

        self.processWord()
        
    def bindRightWord(self):
        """ Bound to the right button. """

	self.bindSetComment()
	self.x = 0

#	self.ne.clear()
        index = self.currentAnnSet.index(self.currentAnn)
        if index == 0:
            return
        try:
            self.currentAnn = self.currentAnnSet[index-1]
        except:
            return

	self.block.clear()
	string = str(self.currentAGIndex+1)
	string += "    (word "
	string += str(index)
	string += ")"
	self.block.setText(string)

        self.processWord()

    def insertAnalyses(self):
        """ Insert all the analyses in the list box. """
        
        self.lb.clear()
        #print ag.GetFeature(self.currentAnn, 'input')
        try:
            lookup = ag.GetFeature(self.currentAnn, 'lookup-word')
        except:
            lookup = None
        self.trans.clear()
        if lookup:
            self.trans.setText(lookup)
            
        self.comments.clear()
	comments = None
        try:
            comments = ag.GetFeature(self.currentAnn, 'comment')
        except:
            comments = None

        if comments:
            self.comments.setText(comments)

	n = 0
	num = 0
	displayList = []
        for i in self.currentAnalysesDict[self.currentAnn]:
            string = ag.GetFeature(i, 'solution')
	    try:
		string += "     : "+ag.GetFeature(i, 'gloss')
		n = ag.GetFeature(i, 'number')
	    except:
		string += " : "+"NO_GLOSS"
		n = '999'
            newstring = self.parseText(string)
	    displayList = displayList + [(newstring, num)]
	    # print(num, newstring)
	    num += 1
	# print(displayList)

	if num == 0:
	    if comments == None:
		comments = "NO_SOLUTION"
		self.comments.setText(comments)
        try:
            sel = ag.GetFeature(self.currentAnn, 'selection')
        except:
	    if num == 1:
		string = ag.GetFeature(self.currentAnalysesDict[self.currentAnn][0], 'solution')
		if (re.search('\/PUNC', string) != None) | (re.search('\/NUM', string) != None ):
		    ag.SetFeature(self.currentAnn, 'selection', self.currentAnalysesDict[self.currentAnn][0])
		    sel = string
		else:
		    sel = None
	    else:
		sel = None

	displayList.sort()
	self.refTable = []
	oldpos = ""
	oldtxt = ""
	for item in displayList:
	    txt = QString()
	    txt = item[0]
	    self.refTable = self.refTable + [(item[1])]
	    self.lb.insertItem(txt)

    def parseText(self, s):
        """ Parse the input string for re-formatted text display. """
	return s

    def stripCase(self, s):
        """ Strip the case ending """
	return s

    def sortKey(self, s):
        """ Parse the input string for re-formatted text display. """
	return s

    def bindGotoBlock(self):
	""" Bound to the goto paragraph button. """

	(p, tag) = QInputDialog.getInteger("GoTo Paragraph", "Please enter the paragraph number:", self.currentAGIndex+1, 1, len(self.agIDs))
	if tag == 0:
	    return
	else:
	    p -= 1
	if p < 0:
	    return
	if p == self.currentAGIndex:
	    return 
	if p >= len(self.agIDs):
	    return
	self.gotoBlock(p)

    def gotoBlock(self, i):
        """ Bound to the goto paragraph button. """ 

	if i == self.currentAGIndex:
	    return

	self.bindStopSegment()
	self.bindSetComment()
        try:
            agID = self.agIDs[self.currentAGIndex+1]
            self.currentAGIndex = i
            self.block.clear()
            self.block.setText(str(self.currentAGIndex+1))
            self.currentAG = agID
            self.processBlock(self.currentAGIndex)
        except:
            pass

    def bindNextBlock(self):
        """ Bound to the next paragraph button. """ 

	self.bindStopSegment()
	self.bindSetComment()

#	self.ne.clear()
        try:
            agID = self.agIDs[self.currentAGIndex+1]
            self.currentAGIndex += 1
            self.block.clear()
            self.block.setText(str(self.currentAGIndex+1))
            self.currentAG = agID
            self.processBlock(self.currentAGIndex)
        except:
            pass
	
    def bindPrevBlock(self):
        """ Bound to the prev. paragraph button. """

	self.bindStopSegment()
	self.bindSetComment()        

#	self.ne.clear()
        if self.currentAGIndex == 0:
            return
        try:
            agID = self.agIDs[self.currentAGIndex-1]
            self.currentAGIndex -= 1
            self.block.clear()
            self.block.setText(str(self.currentAGIndex+1))
            self.currentAG = agID
            self.processBlock(self.currentAGIndex)
        except:
            pass
        
    def preprocessTextFile(self, inString):
        pass
    def loadTextFile(self, filename):
        pass
    def displayText(self, textList, index):
        pass
    def setOutputFilename(self, filename):
        self.outputFilename = filename

class ArabicSelectPOS(SelectPOS):
    """ Arabic-specific operations.  The idea is to generalize the classes
    SelectPOS and SelectPOSMain, and have methods overwritten in inherited
    classes for project-specific operations.  ... When I have time, I will
    work on the rest. """
    
    def displayText(self, textList, index):
        self.e.setText(textList[index])
    
    def loadTextFile(self, filename):
        """
        Read a text file.
        Extract text between the <P> and </P> tags.
        Return them as a tuple of strings.
        """

	try:
            textFile = UTF8Reader(open(filename, 'r'))
	    print(filename + ' opened')
	except:
	    print('source file ' + filename + ' not found')
	    filename = filename.split('/')[-1]
	    try:
		textFile = UTF8Reader(open(filename, 'r'))
		print('source file ' + filename + ' opened')
	    except:
		print('source file ' + filename + ' not found')
		filename = 'sgm/' + filename
		print('Now have to try last one at ' + filename)
		textFile = UTF8Reader(open(filename, 'r'))
		print('source file ' + filename + ' opened')

	out = ()
	while 1:
            line = textFile.readline()
	    # print UTF8Encode(line)
            if not line:
		#for i in out:
		#    print UTF8Encode(i)[0]
		textFile.close()
                return out
            # line = re.sub('&lt;', '<', line)
            # line = re.sub('&gt;', '>', line)
            # line = re.sub('&amp;', '&', line)
            # print "NEW LINE", Encode(line)
            if re.sub('[\s\n\r]', '', line) != '':
                out = out + (line,)
        return out

### MAIN
                
root = QApplication(sys.argv)
font = QFont("clearlyu", 16, 50, 0)

opts, pargs = getopt.getopt(sys.argv[1:], 'o:')

outputFilename = None
for i in opts:
    if i[0] == '-o':
        outputFilename = i[1]
try:
    filename = pargs[0]        
except:
    filename = None

#print "AIF File", filename
#print "Output AIF File", outputFilename

# create main application    
ta = ArabicSelectPOSMain()
if filename:
    ta.main.loadAIFFile(filename)
if outputFilename:
    ta.main.setOutputFilename(outputFilename)
ta.show()

root.connect(root, SIGNAL('lastWindowClosed()'), root, SLOT('quit()'))
root.exec_loop()            





